package com.zdz.redis.test;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import redis.clients.jedis.Jedis;

/**
 * 使用Jedis客户端 java操作redis测试
 * @author zdz
 *
 */
public class JedisTest {
	public static Jedis jedis;
	
	public static void main(String[] args) {
		jedis = new Jedis("localhost");
		jedis.set("foo", "bar");
		String value = jedis.get("foo");
		System.out.println(value);
		String value2 = jedis.get("zdz");
		System.out.println(value2);
		
		testBasicString();
		testCommon();
		/*
		 bar
zhengdz
zdz
zdzjarorwar
大自然
null
[zdz, 大自然]
[jarorwar, zdz, foo, sname, a, name]
[sname, name]
0
-1
2
1
false
OK
null
min
[9, 3, 6, 1]
[1, 3, 6, 9]
[9, 3, 6, 1]

		 */
	}
	
	/**
     * Redis存储常用操作方法
     */
	public static void testCommon() {
        //keys中传入的可以用通配符
        System.out.println(jedis.keys("*")); //返回当前库中所有的key [jarorwar, zdz, foo, a, name]
        System.out.println(jedis.keys("*name"));//返回的key  [sname, name]
        System.out.println(jedis.del("noname"));//删除key为noname的对象  删除成功返回1 删除失败（或者不存在）返回 0
        
        System.out.println(jedis.ttl("sname"));//返回给定key的有效时间，如果是-1则表示永远有效
        jedis.setex("timekey", 4, "min");//通过此方法，可以指定key的存活（有效时间） 时间为秒
        try {
			Thread.sleep(2000);
		} catch (InterruptedException e) {}
        //睡眠2秒后，剩余时间将为<=2
        System.out.println(jedis.ttl("timekey"));   //输出结果为2
        jedis.setex("timekey", 1, "min");        //设为1后，下面再看剩余时间就是1了
        System.out.println(jedis.ttl("timekey"));  //输出结果为1
        System.out.println(jedis.exists("key"));//检查key是否存在
        System.out.println(jedis.rename("timekey","time"));
        System.out.println(jedis.get("timekey"));//因为移除，返回为null
        System.out.println(jedis.get("time")); //因为将timekey 重命名为time 所以可以取得值 min

        //jedis 排序
        //注意，此处的rpush和lpush是List的操作。是一个双向链表（但从表现来看的）
        jedis.del("a");//先清除数据，再加入数据进行测试
        jedis.rpush("a", "1");
        jedis.lpush("a","6");
        jedis.lpush("a","3");
        jedis.lpush("a","9");
        System.out.println(jedis.lrange("a",0,-1));// [9, 3, 6, 1]
        System.out.println(jedis.sort("a")); //[1, 3, 6, 9]  //输入排序后结果
        System.out.println(jedis.lrange("a",0,-1));//排序不影响原来的结果
    }
	
	/**
     * Redis存储初级的字符串
     */
    public static void testBasicString(){
        //-----添加数据----------
        jedis.set("name","zdz");//向key-->name中放入了value-->zdz
        System.out.println(jedis.get("name"));//执行结果：zdz

        //-----修改数据-----------
        //1、在原来基础上修改
        jedis.append("name","jarorwar");   //很直观，类似map 将jarorwar append到已经有的value之后
        System.out.println(jedis.get("name"));//执行结果:zdzjarorwar

        //2、直接覆盖原来的数据
        jedis.set("name","大自然");
        System.out.println(jedis.get("name"));//执行结果：大自然

        //删除key对应的记录
        jedis.del("name");
        System.out.println(jedis.get("name"));//执行结果：null

        /**
         * mset相当于
         * jedis.set("name","zdz");
         * jedis.set("jarorwar","大自然");
         */
        jedis.mset("name","zdz","jarorwar","大自然");
        System.out.println(jedis.mget("name","jarorwar"));
        
        jedis.set("sname","liufeng");
    }

    /**
     * jedis操作Map
     */
    public static void testMap(){
        Map<String,String> user=new HashMap<String,String>();
        user.put("name","zdz");
        user.put("pwd","password");
        jedis.hmset("user",user);
        //取出user中的name，执行结果:[zdz]-->注意结果是一个泛型的List
        //第一个参数是存入redis中map对象的key，后面跟的是放入map中的对象的key，后面的key可以跟多个，是可变参数
        List<String> rsmap = jedis.hmget("user", "name");
        System.out.println(rsmap);

        //删除map中的某个键值
        jedis.hdel("user","pwd");
        System.out.println(jedis.hmget("user", "pwd")); //因为删除了，所以返回的是null
        System.out.println(jedis.hlen("user")); //返回key为user的键中存放的值的个数1
        System.out.println(jedis.exists("user"));//是否存在key为user的记录 返回true
        System.out.println(jedis.hkeys("user"));//返回map对象中的所有key  [pwd, name]
        System.out.println(jedis.hvals("user"));//返回map对象中的所有value  [zdz, password]

        Iterator<String> iter=jedis.hkeys("user").iterator();
        while (iter.hasNext()){
            String key = iter.next();
            System.out.println(key+":"+jedis.hmget("user",key));
        }

    }

    /**
     * jedis操作List
     */
    public static void testList(){
        //开始前，先移除所有的内容
        jedis.del("java framework");
        System.out.println(jedis.lrange("java framework",0,-1));
       //先向key java framework中存放三条数据
       jedis.lpush("java framework","spring");
       jedis.lpush("java framework","struts");
       jedis.lpush("java framework","hibernate");
       //再取出所有数据jedis.lrange是按范围取出，
       // 第一个是key，第二个是起始位置，第三个是结束位置，jedis.llen获取长度 -1表示取得所有
       System.out.println(jedis.lrange("java framework",0,-1));
    }

    /**
     * jedis操作Set
     */
    public static void testSet(){
        //添加
        jedis.sadd("sname","zdz");
        jedis.sadd("sname","jarorwar");
        jedis.sadd("sname","大自然");
        jedis.sadd("sanme","non");
        //移除noname
        jedis.srem("sname","non");
        System.out.println(jedis.smembers("sname"));//获取所有加入的value
        System.out.println(jedis.sismember("sname", "zdz"));//判断 zdz 是否是sname集合的元素
        System.out.println(jedis.srandmember("sname"));//取一个随机数
        System.out.println(jedis.scard("sname"));//返回集合的元素个数
    }

}
